@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Collector.pm,v 1.6 2005/06/21 13:03:26 ssinyagin Exp $
+# $Id: Collector.pm,v 1.8 2005/08/11 12:32:26 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -90,30 +90,41 @@ sub addTarget
 
     $self->{'targets'}{$token}{'type'} = $collector_type;
 
-    my $storage_type = $config_tree->getNodeParam($token, 'storage-type');
-    if( not $Torrus::Collector::storageTypes{$storage_type} )
+    my $storage_types = $config_tree->getNodeParam($token, 'storage-type');
+    foreach my $storage_type ( split( ',', $storage_types ) )
     {
-        Error('Unknown storage type: ' . $storage_type);
-        return;
+        if( not $Torrus::Collector::storageTypes{$storage_type} )
+        {
+            Error('Unknown storage type: ' . $storage_type);
+        }
+        else
+        {
+            my $storage_string = $storage_type . '-storage';
+            if( not exists( $self->{'targets'}{$token}{'storage-types'} ) )
+            {
+                $self->{'targets'}{$token}{'storage-types'} = [];
+            }
+            push( @{$self->{'targets'}{$token}{'storage-types'}},
+                  $storage_type );
+            
+            $self->fetchParams($config_tree, $token, $storage_string);
+        }
     }
 
-    my $storage_string = $storage_type.'-storage';
-    $self->{'targets'}{$token}{'storage-type'} = $storage_type;
-
     # If specified, store the value transformation code
     my $code = $config_tree->getNodeParam($token, 'transform-value');
     if( defined $code )
     {
         $self->{'targets'}{$token}{'transform'} = $code;
     }
-
+    
     # If specified, store the scale RPN
     my $scalerpn = $config_tree->getNodeParam($token, 'collector-scale');
     if( defined $scalerpn )
     {
         $self->{'targets'}{$token}{'scalerpn'} = $scalerpn;
     }
-
+    
     # If specified, store the value map
     my $valueMap = $config_tree->getNodeParam($token, 'value-map');
     if( defined $valueMap and length($valueMap) > 0 )
@@ -133,19 +144,26 @@ sub addTarget
         $self->{'targets'}{$token}{'local'} = {};
     }
 
-    $self->fetchParams($config_tree, $token, $storage_string);
-
     if( ref( $Torrus::Collector::initTarget{$collector_type} ) )
     {
         $ok = &{$Torrus::Collector::initTarget{$collector_type}}($self,
                                                                  $token);
     }
 
-    if( $ok and ref( $Torrus::Collector::initTarget{$storage_string} ) )
+    if( $ok )
     {
-        &{$Torrus::Collector::initTarget{$storage_string}}($self, $token);
+        foreach my $storage_type
+            ( @{$self->{'targets'}{$token}{'storage-types'}} )
+        {
+            my $storage_string = $storage_type . '-storage';
+            if( ref( $Torrus::Collector::initTarget{$storage_string} ) )
+            {
+                &{$Torrus::Collector::initTarget{$storage_string}}($self,
+                                                                   $token);
+            }
+        }
     }
-
+    
     if( not $ok )
     {
         $self->deleteTarget( $token );
@@ -479,10 +497,13 @@ sub setValue
     Debug('Value ' . $value . ' set for ' .
           $self->path($token) . ' TS=' . $timestamp);
 
-    my $proc = $Torrus::Collector::setValue{
-        $self->{'targets'}{$token}{'storage-type'}};
-    
-    &{$proc}( $self, $token, $value, $timestamp, $uptime );
+    foreach my $storage_type
+        ( @{$self->{'targets'}{$token}{'storage-types'}} )
+    {
+        &{$Torrus::Collector::setValue{$storage_type}}( $self, $token,
+                                                        $value, $timestamp,
+                                                        $uptime );
+    }
 }
 
 
