@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SNMP_Params.pm,v 1.2 2005/01/25 13:46:33 ssinyagin Exp $
+# $Id: SNMP_Params.pm,v 1.3 2005/11/22 16:31:18 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::SNMP_Params;
@@ -26,11 +26,23 @@ my %validatorLeafParams =
     (
      'snmp-host'          => undef,
      'snmp-port'          => undef,
-     '+domain-name'       => undef,
-     'snmp-community'     => undef,
+     '+domain-name'       => undef,     
      'snmp-object'        => undef,
-     'snmp-version'       => { '1'  => undef,
-                               '2c' => undef },
+     'snmp-version'       => { '1'  => { 'snmp-community'     => undef },
+                               '2c' => { 'snmp-community'     => undef },
+                               '3'  => {
+                                   'snmp-username' => undef,
+                                   '+snmp-authkey' => undef,
+                                   '+snmp-authpassword' => undef,
+                                   '+snmp-authprotocol' => {
+                                       'md5' => undef,
+                                       'sha' => undef },
+                                   '+snmp-privkey' => undef,
+                                   '+snmp-privpassword' => undef,
+                                   '+snmp-privprotocol' => {
+                                       'des'       => undef,
+                                       'aes128cfb' => undef,
+                                       '3desede'   => undef } } },
      'snmp-timeout'       => undef,
      'snmp-retries'       => undef,
      'snmp-oids-per-pdu'  => undef,
@@ -54,6 +66,13 @@ my %admInfoLeafParams =
      'snmp-port'          => undef,
      'domain-name'        => undef,
      'snmp-community'     => undef,
+     'snmp-username'      => undef,
+     'snmp-authkey'       => undef,
+     'snmp-authpassword'  => undef,
+     'snmp-authprotocol'  => undef,
+     'snmp-privkey'       => undef,
+     'snmp-privpassword'  => undef,
+     'snmp-privprotocol'  => undef,
      'snmp-object'        => undef,
      'snmp-version'       => undef,
      'snmp-timeout'       => undef,
@@ -70,6 +89,13 @@ my %admInfoParamCategories =
      'snmp-port'          => 'SNMP',
      'domain-name'        => 'SNMP',
      'snmp-community'     => 'SNMP',
+     'snmp-username'      => 'SNMP',
+     'snmp-authkey'       => 'SNMP',
+     'snmp-authpassword'  => 'SNMP',
+     'snmp-authprotocol'  => 'SNMP',
+     'snmp-privkey'       => 'SNMP',
+     'snmp-privpassword'  => 'SNMP',
+     'snmp-privprotocol'  => 'SNMP',     
      'snmp-object'        => 'SNMP',
      'snmp-version'       => 'SNMP',
      'snmp-timeout'       => 'SNMP',
