@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Writer.pm,v 1.3 2005/07/18 15:53:24 ssinyagin Exp $
+# $Id: Writer.pm,v 1.6 2006/02/23 12:07:43 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 #
@@ -28,20 +28,25 @@ our @ISA=qw(Torrus::ConfigTree);
 
 use Torrus::Log;
 use Torrus::TimeStamp;
-
+use Torrus::ServiceID;
+    
 use strict;
 use Digest::MD5 qw(md5); # needed as hash function
 
 # Global list of parameters requiring space removal
 my @remspace =
-    (qw(monitor action launch-when rpn-expr tokenset-member setenv-params
-        setenv-dataexpr ds-names value-map rrgraph-views print-cf hrules));
+    (qw(storage-type monitor action launch-when rpn-expr tokenset-member
+        setenv-params setenv-dataexpr ds-names value-map rrgraph-views
+        print-cf hrules display-rpn-expr));
 
 foreach my $param ( @remspace )
 {
     $Torrus::ConfigTree::Writer::remove_space{$param} = 1;
 }
 
+# instance of Torrus::ServiceID object, if needed
+my $srvIdParams;
+
 
 sub new
 {
@@ -471,6 +476,42 @@ sub postProcessNodes
                         }
                     }
                 }
+
+                my $storagetypes =
+                    $self->getNodeParam( $token, 'storage-type' );
+                foreach my $stype ( split(',', $storagetypes) )
+                {
+                    if( $stype eq 'ext' )
+                    {
+                        my $tree = $self->treeName();
+                        if( not defined( $srvIdParams ) )
+                        {
+                            $srvIdParams =
+                                new Torrus::ServiceID( -WriteAccess => 1 );
+                            $srvIdParams->cleanAllForTree( $tree );
+                        }
+
+                        my $serviceid =
+                            $self->getNodeParam($token, 'ext-service-id');
+                        if( $srvIdParams->idExists( $serviceid ) )
+                        {
+                            Error('Duplicate ServiceID: ' . $serviceid);
+                            $ok = 0;
+                        }
+                        else
+                        {
+                            my $params = {
+                                'tree' => $tree,
+                                'token' => $token,
+                                'dstype' => $self->getNodeParam($token,
+                                                                'ext-dstype'),
+                                'units' => $self->getNodeParam
+                                    ($token, 'ext-service-units') };
+
+                            $srvIdParams->add( $serviceid, $params );
+                        }
+                    }
+                }
             }
         }
         else
