@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DevDiscover.pm,v 1.16 2005/07/22 15:57:51 ssinyagin Exp $
+# $Id: DevDiscover.pm,v 1.21 2006/07/20 09:34:28 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Core SNMP device discovery module
@@ -42,7 +42,6 @@ BEGIN
 our @requiredParams =
     (
      'snmp-port',
-     'snmp-community',
      'snmp-version',
      'snmp-timeout',
      'snmp-retries',
@@ -64,6 +63,7 @@ our @copyParams =
       'collector-timeoffset-min',
       'collector-timeoffset-max',
       'collector-timeoffset-step',
+      'comment',
       'domain-name',
       'monitor-period',
       'monitor-timeoffset',
@@ -71,6 +71,13 @@ our @copyParams =
       'snmp-port',
       'snmp-community',
       'snmp-version',
+      'snmp-username',
+      'snmp-authkey',
+      'snmp-authpassword',
+      'snmp-authprotocol',
+      'snmp-privkey',
+      'snmp-privpassword',
+      'snmp-privprotocol',
       'snmp-timeout',
       'snmp-retries',
       'snmp-oids-per-pdu',
@@ -144,10 +151,47 @@ sub discover
     }
 
     my %snmpargs;
-    foreach my $arg ( qw(-port -version -community -timeout -retries) )
+    my $community;
+    my $version = $devdetails->param( 'snmp-version' );
+    $snmpargs{'-version'} = $version;    
+
+    foreach my $arg ( qw(-port -timeout -retries) )
     {
         $snmpargs{$arg} = $devdetails->param( 'snmp' . $arg );
     }
+    
+    if( $version eq '1' or $version eq '2c' )
+    {
+        $community = $devdetails->param( 'snmp-community' );
+        if( not defined( $community ) )
+        {
+            Error('Required parameter not defined: snmp-community');
+            return 0;
+        }
+        $snmpargs{'-community'} = $community;
+    }
+    elsif( $version eq '3' )        
+    {
+        foreach my $arg ( qw(-username -authkey -authpassword -authprotocol
+                             -privkey -privpassword -privprotocol) )
+        {
+            if( defined $devdetails->param( 'snmp' . $arg ) )
+            {
+                $snmpargs{$arg} = $devdetails->param( 'snmp' . $arg );
+            }
+        }
+        $community = $snmpargs{'-username'};
+        if( not defined( $community ) )
+        {
+            Error('Required parameter not defined: snmp-user');
+            return 0;
+        }        
+    }
+    else
+    {
+        Error('Illegal value for snmp-version parameter: ' . $version);
+        return 0;
+    }
 
     my $hostname = $devdetails->param('snmp-host');
     my $domain = $devdetails->param('domain-name');
@@ -159,8 +203,6 @@ sub discover
     $snmpargs{'-hostname'} = $hostname;
 
     my $port = $snmpargs{'-port'};
-    my $community = $snmpargs{'-community'};
-
     Debug('Discovering host: ' . $hostname . ':' . $port . ':' . $community);
 
     my ($session, $error) =
@@ -250,7 +292,11 @@ sub discover
             $legend .= $legendValues{$key}{'name'} . ':' . $text . ';';
         }
     }
-    $data->{'param'}{'legend'} = $legend;
+    
+    if( $devdetails->param('suppress-legend') ne 'yes' )
+    {
+        $data->{'param'}{'legend'} = $legend;
+    }
 
     # some parameters need just one-to-one copying
 
@@ -340,6 +386,28 @@ sub discover
     }
     push( @{$self->{'devdetails'}{$subtree}}, $devdetails );
 
+    my $define_tokensets = $devdetails->param('define-tokensets');
+    if( defined( $define_tokensets ) and length( $define_tokensets ) > 0 )
+    {
+        foreach my $pair ( split(/\s*;\s*/, $define_tokensets ) )
+        {
+            my( $tset, $description ) = split( /\s*:\s*/, $pair );
+            if( $tset !~ /^[a-z][a-z0-9-_]*$/ )
+            {
+                Error('Invalid name for tokenset: ' . $tset);
+                $ok = 0;
+            }
+            elsif( length( $description ) == 0 )
+            {
+                Error('Missing description for tokenset: ' . $tset);
+                $ok = 0;
+            }
+            else
+            {
+                $self->{'define-tokensets'}{$tset} = $description;
+            }
+        }
+    }
     return $ok;
 }
 
@@ -350,7 +418,7 @@ sub buildConfig
     my $cb = shift;
 
     my $reg = \%Torrus::DevDiscover::registry;
-
+        
     foreach my $subtree ( sort keys %{$self->{'devdetails'}} )
     {
         # Chop the first and last slashes
@@ -423,6 +491,16 @@ sub buildConfig
             $cb->{'statistics'}{'hosts'}++;
         }
     }
+
+    if( defined( $self->{'define-tokensets'} ) )
+    {
+        my $tsetsNode = $cb->startTokensets();
+        foreach my $tset ( sort keys %{$self->{'define-tokensets'}} )
+        {
+            $cb->addTokenset( $tsetsNode, $tset, {
+                'comment' => $self->{'define-tokensets'}{$tset} } );
+        }
+    }
 }
 
 
