@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoGeneric.pm,v 1.7 2005/07/22 15:57:52 ssinyagin Exp $
+# $Id: CiscoGeneric.pm,v 1.9 2006/02/13 14:10:32 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Common Cisco MIBs, supported by many IOS and CatOS devices
@@ -43,6 +43,7 @@ our %oiddef =
      'ciscoEnvMonTemperatureStatusValue' => '1.3.6.1.4.1.9.9.13.1.3.1.3',
      'ciscoEnvMonTemperatureThreshold'   => '1.3.6.1.4.1.9.9.13.1.3.1.4',
      'ciscoEnvMonTemperatureStatusState' => '1.3.6.1.4.1.9.9.13.1.3.1.6',
+     'ciscoEnvMonSupplyState'            => '1.3.6.1.4.1.9.9.13.1.5.1.3',
 
      # CISCO-ENHANCED-MEMPOOL-MIB
      'cempMemPoolName'                   => '1.3.6.1.4.1.9.9.221.1.1.1.1.3',
@@ -121,6 +122,33 @@ sub discover
         }
     }
 
+    if( $devdetails->param('CiscoGeneric::disable-psupplies') ne 'yes' )
+    {
+        # Check if power supply status is supported
+
+        my $oidSupply = $dd->oiddef('ciscoEnvMonSupplyState');
+
+        my $supplyTable = $session->get_table( -baseoid => $oidSupply );
+        if( defined( $supplyTable ) )
+        {
+            $devdetails->setCap('ciscoPowerSupplies');
+            $data->{'ciscoPowerSupplies'} = [];
+            
+            my $prefixLen = length( $oidSupply ) + 1;
+            while( my( $oid, $val ) = each %{$supplyTable} )
+            {
+                # Extract the supply index from OID
+                my $sIndex = substr( $oid, $prefixLen );
+                
+                #check if the value is not notPresent(5)
+                if( $val != 5 )
+                {
+                    push( @{$data->{'ciscoPowerSupplies'}}, $sIndex );
+                }
+            }
+        }
+    }
+    
     if( $devdetails->param('CiscoGeneric::disable-memory-pools') ne 'yes' )
     {
         my $eMemPool =
@@ -207,6 +235,18 @@ sub discover
 
             $data->{'ciscoCpuStats'} = {};
 
+            # Find multiple CPU entries pointing to the same Phy index
+            my %phyReferers = ();            
+            foreach my $INDEX
+                ( $devdetails->
+                  getSnmpIndices($dd->oiddef('cpmCPUTotalPhysicalIndex') ) )
+            {
+                my $phyIndex = $devdetails->
+                    snmpVar($dd->oiddef('cpmCPUTotalPhysicalIndex') .
+                            '.' . $INDEX );
+                $phyReferers{$phyIndex}++;
+            }
+                
             foreach my $INDEX
                 ( $devdetails->
                   getSnmpIndices($dd->oiddef('cpmCPUTotalPhysicalIndex') ) )
@@ -235,10 +275,16 @@ sub discover
                 $cpuNick =~ s/\W/_/g;
                 $cpuNick =~ s/_+/_/g;
 
+                if( $phyReferers{$phyIndex} > 1 )
+                {
+                    $phyDescr .= ' (' . $INDEX . ')';
+                }
+                
                 $data->{'ciscoCpuStats'}{$INDEX} = {
                     'phy-index'  => $phyIndex,
                     'phy-name'   => $phyName,
                     'phy-descr'  => $phyDescr,
+                    'phy-referers' => $phyReferers{$phyIndex},
                     'cpu-nick'   => $cpuNick };
                 
                 if( $devdetails->hasOID( $dd->oiddef('cpmCPUTotal1minRev') .
@@ -340,6 +386,43 @@ sub buildConfig
         }
     }
 
+    # Power supplies
+
+    if( $devdetails->hasCap('ciscoPowerSupplies') )
+    {
+        # Create a subtree for the power supplies
+        my $subtreeName = 'Power_Supplies';
+
+        my $param = {'comment' => 'Power supplies status',
+                     'precedence' => -600};
+        my $templates = [];
+                
+        $param->{'data-file'} = '%snmp-host%_power.rrd';
+
+        my $monitor = $devdetails->param('CiscoGeneric::power-monitor');
+        if( length( $monitor ) > 0 )
+        {
+            $param->{'monitor'} = $monitor;
+        }
+
+        my $subtreeNode = $cb->addSubtree( $devNode, $subtreeName,
+                                           $param, $templates );
+        
+        foreach my $sIndex ( sort {$a<=>$b} @{$data->{'ciscoPowerSupplies'}} )
+        {
+            my $leafName = sprintf( 'power_%.2d', $sIndex );
+
+            my $param = {
+                'power-index'       => $sIndex
+                };
+
+            my $templates = ['CiscoGeneric::cisco-power-supply'];
+
+            $cb->addLeaf( $subtreeNode, $leafName, $param, $templates );
+        }
+    }
+
+    
     # Memory Pools
 
     if( $devdetails->hasCap('cempMemPool') or
@@ -450,10 +533,21 @@ sub buildConfig
             my $cpu = $data->{'ciscoCpuStats'}{$INDEX};
 
             my $param = {
-                'entity-phy-index' => $cpu->{'phy-index'},
                 'comment' => $cpu->{'phy-descr'} . ' in ' . $cpu->{'phy-name'}
                 };
 
+            # On newer dual-CPU routers, several (two seen) CPU entries
+            # refer to the same physical entity. For such entries,
+            # we map them directly to cpmCPUTotalTable index.
+            if( $cpu->{'phy-referers'} > 1 )
+            {
+                $param->{'cisco-cpu-indexmap'} = $INDEX;
+            }
+            else
+            {
+                $param->{'entity-phy-index'} = $cpu->{'phy-index'};
+            }
+            
             my @templates;
 
             if( $cpu->{'stats-type'} eq 'revised' )
