@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2670_DOCS_IF.pm,v 1.9 2005/06/08 08:08:37 ssinyagin Exp $
+# $Id: RFC2670_DOCS_IF.pm,v 1.10 2005/09/01 20:29:47 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # DOCSIS interface statistics
@@ -157,6 +157,18 @@ sub buildConfig
         if( scalar( @{$data->{$category}} ) > 0 and
             scalar( @{$data->{'docsConfig'}{$category}{'templates'}} ) > 0 )
         {
+            # Count non-excluded interfaces
+            my $updatedInterfaceList = [];
+            foreach my $ifIndex ( @{$data->{$category}} )
+            {
+                my $interface = $data->{'interfaces'}{$ifIndex};
+                next if $interface->{'excluded'};
+                push( @{$updatedInterfaceList}, $ifIndex );
+            }
+            $data->{$category} = $updatedInterfaceList;
+            
+            next if scalar( @{$data->{$category}} ) == 0;
+
             my $subtreeNode =
                 $cb->addSubtree( $devNode,
                                  $data->{'docsConfig'}{$category}{
