@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2863_IF_MIB.pm,v 1.20 2005/07/22 15:57:53 ssinyagin Exp $
+# $Id: RFC2863_IF_MIB.pm,v 1.28 2006/05/17 12:37:25 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Standard IF_MIB discovery, which should apply to most devices
@@ -139,10 +139,14 @@ sub discover
     foreach my $ifIndex
         ( $devdetails->getSnmpIndices( $dd->oiddef('ifDescr') ) )
     {
-        if( ( $Torrus::DevDiscover::listAdminDownInterfaces or
+        if( ( $devdetails->param('RFC2863_IF_MIB::list-admindown-interfaces')
+              eq 'yes'
+              or
               $devdetails->snmpVar($dd->oiddef('ifAdminStatus') .
                                    '.' . $ifIndex) == 1 ) and
-            ( $Torrus::DevDiscover::listNotPresentInterfaces or
+            ( $devdetails->param('RFC2863_IF_MIB::list-notpresent-interfaces')
+              eq 'yes'
+              or
               $devdetails->snmpVar($dd->oiddef('ifOperStatus') .
                                    '.' . $ifIndex) != 6 ) )
         {
@@ -377,6 +381,11 @@ sub buildConfig
     uniqueEntries( $devdetails, $data->{'nameref'}{'ifNick'} );
     uniqueEntries( $devdetails, $data->{'nameref'}{'ifSubtreeName'} );
 
+    if( $devdetails->param('RFC2863_IF_MIB::noout') eq 'yes' )
+    {
+        return;
+    }
+    
     # Build interface parameters
 
     my $nInterfaces = 0;
@@ -447,6 +456,8 @@ sub buildConfig
 
     # tokenset member interfaces of the form
     # Format: tset:intf,intf; tokenset:intf,intf;
+    # Format for global parameter:
+    #     tset:host/intf,host/intf; tokenset:host/intf,host/intf;
     my %tsetMember;
     my %tsetMemberApplied;
     my $tsetMembership =
@@ -458,11 +469,45 @@ sub buildConfig
             my ($tset, $list) = split( /\s*:\s*/, $memList );
             foreach my $intfName ( split( /\s*,\s*/, $list ) )
             {
-                $tsetMember{$intfName}{$tset} = 1;
+                if( $intfName =~ /\// )
+                {
+                    my( $host, $intf ) = split( '/', $intfName );
+                    if( $host eq $devdetails->param('snmp-host') )
+                    {
+                        $tsetMember{$intf}{$tset} = 1;
+                    }
+                }
+                else
+                {
+                    $tsetMember{$intfName}{$tset} = 1;
+                }
             }
         }
     }
 
+        
+    # External storage serviceid assignment
+    my $extSrv =
+        $devdetails->param('RFC2863_IF_MIB::external-serviceid');
+    my %extStorage;
+    if( defined( $extSrv ) and length( $extSrv ) > 0 )
+    {
+        foreach my $srvDef ( split( /\s*,\s*/, $extSrv ) )
+        {
+            my ( $serviceid, $intfName, $direction ) =
+                split( /\s*:\s*/, $srvDef );
+            if( $direction eq 'Both' )
+            {
+                $extStorage{$intfName}{'In'} = $serviceid . '_IN';
+                $extStorage{$intfName}{'Out'} = $serviceid . '_OUT';
+            }
+            else
+            {
+                $extStorage{$intfName}{$direction} = $serviceid;
+            }
+        }
+    }
+    
     # interface-level parameters to copy
     my @intfCopyParams = ();
     my $copyParams = $devdetails->param('RFC2863_IF_MIB::copy-params');
@@ -478,8 +523,14 @@ sub buildConfig
     {
         $subtreeName = 'Interface_Counters';
     }
+    my $subtreeParams = {};
+    my $subtreeComment = $devdetails->param('RFC2863_IF_MIB::subtree-comment');
+    if( length( $subtreeComment ) > 0 )
+    {
+        $subtreeParams->{'comment'} = $subtreeComment;
+    }
     my $countersNode =
-        $cb->addSubtree( $devNode, $subtreeName, undef,
+        $cb->addSubtree( $devNode, $subtreeName, $subtreeParams,
                          ['RFC2863_IF_MIB::rfc2863-ifmib-subtree'] );
     
     foreach my $ifIndex ( sort {$a<=>$b} keys %{$data->{'interfaces'}} )
@@ -496,6 +547,7 @@ sub buildConfig
         {
             if( not $onlyName{$subtreeName} )
             {
+                $interface->{'excluded'} = 1;
                 $nExplExcluded++;
                 next;
             }
@@ -503,6 +555,7 @@ sub buildConfig
         
         if( $excludeName{$subtreeName} )
         {
+            $interface->{'excluded'} = 1;
             $nExplExcluded++;
             next;
         }
@@ -664,7 +717,24 @@ sub buildConfig
                 $interface->{'childCustomizations'}->{'InOut_bps'}->{
                     'tokenset-member'} = $tsetList;
                 $tsetMemberApplied{$subtreeName} = 1;
-            }            
+            }
+
+            if( defined( $extStorage{$subtreeName} ) )
+            {
+                foreach my $dir ( 'In', 'Out' )
+                {
+                    if( defined( $extStorage{$subtreeName}{$dir} ) )
+                    {
+                        $interface->{'childCustomizations'}->{
+                            'Bytes_' . $dir}->{'storage-type'} = 'rrd,ext';
+                        $interface->{'childCustomizations'}->{
+                            'Bytes_' . $dir}->{'ext-service-id'} =
+                                $extStorage{$subtreeName}{$dir};
+                        $interface->{'childCustomizations'}->{
+                            'Bytes_' . $dir}->{'ext-service-units'} = 'bytes';
+                    }
+                }
+            }
             
             my $intfNode =
                 $cb->addSubtree( $countersNode, $subtreeName,
@@ -849,30 +919,42 @@ sub checkSelectorAttribute
 
     my $data = $devdetails->data();
     my $interface = $data->{'interfaces'}{$object};
-
-    my $value;
-    my $operator = '=~';
     
-    if( $attr eq 'ifSubtreeName' )
+    if( $attr =~ /^ifSubtreeName\d*$/ )
     {
-        $value = $interface->{$data->{'nameref'}{'ifSubtreeName'}};
-    }
-    elsif( $attr eq 'ifComment' )
-    {
-        $value = $interface->{$data->{'nameref'}{'ifComment'}};
-    }
-    elsif( $attr eq 'ifType' )
-    {
-        $value = $interface->{'ifType'};
-        $operator = '==';
+        my $value = $interface->{$data->{'nameref'}{'ifSubtreeName'}};
+        my $match = 0;
+        foreach my $chkexpr ( split( /\s+/, $checkval ) )
+        {
+            if( $value =~ $chkexpr )
+            {
+                $match = 1;
+                last;
+            }
+        }
+        return $match;        
     }
     else
     {
-        Error('Unknown RFC2863_IF_MIB selector attribute: ' . $attr);
-        $value = '';
-    }
+        my $value;
+        my $operator = '=~';
+        if( $attr eq 'ifComment' )
+        {
+            $value = $interface->{$data->{'nameref'}{'ifComment'}};
+        }
+        elsif( $attr eq 'ifType' )
+        {
+            $value = $interface->{'ifType'};
+            $operator = '==';
+        }
+        else
+        {
+            Error('Unknown RFC2863_IF_MIB selector attribute: ' . $attr);
+            $value = '';
+        }
 
-    return eval( '$value' . ' ' . $operator . '$checkval' ) ? 1:0;
+        return eval( '$value' . ' ' . $operator . '$checkval' ) ? 1:0;
+    }
 }
 
 
